// Copyright Roger Edgren 2023.
// You may use the code in any way whatsoever and for whatever reasons,
// though I would appreciate it if you didn't remove this header.
using System;
using System.Reflection;
using UnityEngine;


public class DebugConsoleCommand 
{
    private string _methodName;             // the command name, i.e. the method name in some class
    private MethodInfo _methodInfo;         // the method info that correspond to the method name in the command
    private object _classObject;            // the instantiated object whose method will be called
    private ParameterInfo[] _parameters;    // the parameters that needs to be passed to the method
    private int _requiredParameterCount;


    public DebugConsoleCommand(string methodName, object classObject)
    {
        _methodName = methodName;
        _classObject = classObject;
        _requiredParameterCount = 0;
    }


    public DebugConsoleCommand Init()
    {
        // try to find the method in the instantiated object
        Type type = _classObject.GetType();
        MethodInfo[] methods = type.GetMethods();
        for(int i = 0, count = methods.Length; i < count; i++)
        {
            MethodInfo methodInfo = methods[i];
            if (_methodName == methodInfo.Name)
            {
                _methodInfo = methodInfo;
                break;
            }
        }

        if(_methodInfo == null )
        {
            Debug.LogWarning("DebugConsoleCommand.Init: Method: " + _methodName + " not found in " + _classObject.ToString() + ". Command will not be available.");
            return null;
        }

        // get the parameters for the method
        _parameters = _methodInfo.GetParameters();
        _requiredParameterCount = _parameters.Length;
        return this;
    }


    // tries to execute the given command with the supplied parameters. Returns empty string on success, or error string on fail.
    public string TryExecuteCommand(string[] givenParameters)
    {
        // go through each parameter (if any) and try to cast it to the required type. index 0 is the command token so we skip that.
        int givenParameterCount = givenParameters.Length - 1;
        if (givenParameterCount != _requiredParameterCount)
            return GetSyntaxErrorString(); // wrong parameter count for method

        object[] commandParameters = new object[_requiredParameterCount];

        for (int i = 0, count = _parameters.Length; i < count; i++)
        {
            ParameterInfo parameterInfo = _parameters[i];
            string givenParameter = givenParameters[i + 1];

            if (parameterInfo.ParameterType == typeof(string))
                // string
                commandParameters[i] = givenParameter;
            else if (parameterInfo.ParameterType == typeof(int))
            {
                // int
                int parameterAsInt = 0;
                if (!int.TryParse(givenParameter, out parameterAsInt))
                    return GetSyntaxErrorString();
                commandParameters[i] = parameterAsInt;
            }
            else if (parameterInfo.ParameterType == typeof(bool))
            {
                // bool
                bool parameterAsBool = true;
                if (!bool.TryParse(givenParameter, out parameterAsBool))
                    return GetSyntaxErrorString();
                commandParameters[i] = parameterAsBool;
            }
            else if (parameterInfo.ParameterType == typeof(float))
            {
                // float
                float parameterAsFloat = 0.0f;
                if (!float.TryParse(givenParameter, out parameterAsFloat))
                    return GetSyntaxErrorString();
                commandParameters[i] = parameterAsFloat;
            }
        }

        _methodInfo.Invoke(_classObject, commandParameters);
        return string.Empty;
    }


    public string GetSyntaxErrorString()
    {
        string errorString = _methodName + ": wrong syntax. Usage: " + GetUsageString();
        return errorString;
    }


    public string GetUsageString()
    {
        string usageString = _methodName  + " " + GetParametersAsString();
        return usageString;
    }


    public string GetParametersAsString()
    {
        string parametersString = string.Empty;
        for (int i = 0, count = _parameters.Length; i < count; i++)
        {
            ParameterInfo parameterInfo = _parameters[i];
            parametersString += "(" + parameterInfo.ParameterType.ToString() + ") ";
        }
        return parametersString;
    }

}
