// Copyright Roger Edgren 2023.
// You may use the code in any way whatsoever and for whatever reasons,
// though I would appreciate it if you didn't remove this header.
using System.Collections.Generic;
using UnityEngine;


public class DebugConsoleManager : MonoBehaviour
{
    [Header("Debug Console Manager properties")]
    [Tooltip("The debug console UI game object")]
    public GameObject _debugConsoleUIObject;

    private DebugConsoleUIHandler _debugConsoleUIHandler;


    // a dictionary that maps a method string to a console command. We use reflection to find and call the method from this string.
    private Dictionary<string, DebugConsoleCommand> _consoleCommands = new Dictionary<string, DebugConsoleCommand>();


    private void Awake()
    {
        // find the UI handler for the debug console and set it up
        _debugConsoleUIHandler = _debugConsoleUIObject.GetComponent<DebugConsoleUIHandler>();
        _debugConsoleUIHandler.Init();
        _debugConsoleUIHandler._onSubmitCommand += ParseCommand;
        _debugConsoleUIObject.SetActive(false);
    }


    public void AddConsoleCommand(string methodName, object classObject)
    {
        if (_consoleCommands.ContainsKey(methodName))
            return; // already in the dictionary

        DebugConsoleCommand debugConsoleCommand = new DebugConsoleCommand(methodName, classObject).Init();
        if (debugConsoleCommand != null)
            _consoleCommands.Add(methodName, debugConsoleCommand);
    }


    private void ParseCommand(string command)
    {
        if (command == string.Empty)
            return;

        if (command == "?" || command.ToLower() == "help")
        {
            // show all commands
            ShowAllCommands();
            return;
        }

        string[] stringTokens = command.Split(new char[] { ' ' });
        string methodName = stringTokens[0];
        if (!_consoleCommands.ContainsKey(methodName))
        {
            _debugConsoleUIHandler.AddToOutput("Command '" + methodName + "' not found. Type '?' or 'help' to list all commands.");
            return;
        }

        DebugConsoleCommand debugConsoleCommand = _consoleCommands[methodName];
        string errorString = debugConsoleCommand.TryExecuteCommand(stringTokens);
        if (errorString != string.Empty)
            _debugConsoleUIHandler.AddToOutput(errorString); // show error from command execution failure
    }


    public void AddToOutput(string output)
    {
        _debugConsoleUIHandler.AddToOutput(output);
    }


    private void ShowAllCommands()
    {
        foreach(KeyValuePair<string, DebugConsoleCommand> keyValuePair in _consoleCommands)
        {
            DebugConsoleCommand command = keyValuePair.Value;
            _debugConsoleUIHandler.AddToOutput(command.GetUsageString());
        }
    }


    public void Update()
    {
        if (Debug.isDebugBuild)
        {
            if (Input.GetKeyDown(KeyCode.F12))
            {
                if (_debugConsoleUIObject.activeSelf)
                    _debugConsoleUIObject.SetActive(false);
                else
                {
                    _debugConsoleUIObject.SetActive(true);
                    _debugConsoleUIHandler.SetInputFieldFocus();
                }
            }
            else if (_debugConsoleUIObject.activeSelf)
            {
                if (Input.GetKeyDown(KeyCode.UpArrow))
                    _debugConsoleUIHandler.ScrollCommandHistory(true);
                else if (Input.GetKeyDown(KeyCode.DownArrow))
                    _debugConsoleUIHandler.ScrollCommandHistory(false);
                else if (Input.GetKeyDown(KeyCode.Tab))
                    _debugConsoleUIHandler.AutoCompleteText(_consoleCommands);
            }
        }
    }

}
