﻿// Copyright Roger Edgren 2023.
// You may use the code in any way whatsoever and for whatever reasons,
// though I would appreciate it if you didn't remove this header.
using UnityEngine;
using TMPro;
using UnityEngine.Events;
using System.Collections.Generic;


class DebugConsoleUIHandler : MonoBehaviour
{

    [Header("Command Line UI Handler properties")]
    [Tooltip("The input field where the user enters commands")]
    public TMP_InputField _inputField;

    [Tooltip("The output line prefab")]
    public GameObject _outputLinePrefab;

    [Tooltip("The number of output lines")]
    public int _outputLinesCount = 38;

    [Tooltip("The output lines parent")]
    public Transform _outputLinesParent;

    [Tooltip("Show entered commands in output?")]
    public bool _showEnteredCommandsInOutput = false;

    public UnityAction<string> _onSubmitCommand;

    private List<TextMeshProUGUI> _outputLines = new List<TextMeshProUGUI>();
    private List<string> _commandHistory = new List<string>();
    private int _commandHistoryIndex;


    public void Init()
    {
        for (int i = 0; i < _outputLinesCount; i++)
        {
            GameObject outputLineObject = Instantiate(_outputLinePrefab);
            outputLineObject.transform.SetParent(_outputLinesParent, false);
            TextMeshProUGUI outputLine = outputLineObject.GetComponent<TextMeshProUGUI>();
            _outputLines.Add(outputLine);
        }

        _commandHistoryIndex = 0;
        for (int i = 0; i < _outputLinesCount; i++)
            _outputLines[i].text = string.Empty;
    }


    // called from input field (OnEndEdit)
    public void OnSubmitCommand()
    {
        if (Input.GetKeyDown(KeyCode.Escape) || Input.GetKeyDown(KeyCode.F12))
        {
            // OnEndEdit triggers on the escape key, and we don't want that
            SetInputFieldFocus();
            return; 
        }
            
        string command = _inputField.text;
        _onSubmitCommand?.Invoke(command);
        _commandHistory.Insert(0, command);
        _commandHistoryIndex = 0;
        _inputField.text = string.Empty;

        if (_showEnteredCommandsInOutput)
            AddToOutput(command);

        SetInputFieldFocus();
    }


    public void ScrollCommandHistory(bool scrollUp)
    {
        int commandHistoryCount = _commandHistory.Count;
        if (commandHistoryCount == 0)
            return;

        if (scrollUp)
        {
            _inputField.text = _commandHistory[_commandHistoryIndex];
            _commandHistoryIndex++;
            if (_commandHistoryIndex >= commandHistoryCount)
                _commandHistoryIndex = 0;
        }
        else
        {
            _commandHistoryIndex--;
            if (_commandHistoryIndex < 0)
                _commandHistoryIndex = commandHistoryCount - 1;
            _inputField.text = _commandHistory[_commandHistoryIndex];
        }
        _inputField.caretPosition = _inputField.text.Length;
    }


    public void AddToOutput(string output)
    {
        // scroll output lines
        for (int i = _outputLinesCount - 1; i > 0; i--)
            _outputLines[i].text = _outputLines[i - 1].text;

        TextMeshProUGUI currentOutputLine = _outputLines[0];
        currentOutputLine.text = output;
    }


    public void SetInputFieldFocus()
    {
        _inputField.ActivateInputField();
        _inputField.caretPosition = _inputField.text.Length;
    }


    public void AutoCompleteText(Dictionary<string, DebugConsoleCommand> consoleCommands)
    {
        string inputFieldText = _inputField.text;
        int inputFieldTextlength = inputFieldText.Length;
        if (inputFieldTextlength == 0)
            return; // early out

        foreach (KeyValuePair<string, DebugConsoleCommand> keyValuePair in consoleCommands)
        {
            string command = keyValuePair.Key;
            if (command.Length < inputFieldTextlength)
                continue;

            if (inputFieldText == command.Substring(0, inputFieldTextlength))
            {
                _inputField.text = command;
                _inputField.caretPosition = _inputField.text.Length;
                break;
            }
        }
    }

}
