﻿// Copyright Roger Edgren 2023.
// You may use the code in any way whatsoever and for whatever reasons,
// though I would appreciate it if you didn't remove this header.
using UnityEngine;

public static class Util 
{
    public const float DOUBLE_PI = 6.2831853f;
    public const float FLT_EPSILON = 0.00001f;
    public const float MAX_DISTANCE = 999999.0f;
    public const float MIN_SPEED = 0.1f;


    // quadratic formula solver: ax^2 + bx + c = 0 => x = -b +- sqrt(b^2 - 4ac) / 2a
    public static void QuadraticSolver(float a, float b, float c, out float x1, out float x2)
    {
        x1 = 0;
        x2 = 0;
        float preRoot = b * b - 4.0f * a * c;
        // if a == 0 it's a linear formula, not a quadratic one
        if (a == 0)
        {
            x1 = -c / b;
            x2 = x1;
        }
        else if (preRoot > 0)
        {
            x1 = (-b + Mathf.Sqrt(preRoot)) / (2.0f * a);
            x2 = (-b - Mathf.Sqrt(preRoot)) / (2.0f * a);
        }
    }


    // Get an interception position given a tracker's position and its speed, and the target's position and velocity.
    // The predicted interception position is targetPos + targetVelocity * timeToInterception, where timeToInterception is the unknown.
    // The function returns false when the target's speed is greater than the tracker's speed and the target is moving away from the tracker.
    // In this case the tracker will not be able to intercept the target.
    // Source: https://www.codeproject.com/articles/990452/interception-of-two-moving-objects-in-d-space
    public static bool GetInterceptPosition(Vector3 trackerPos, Vector3 targetPos, Vector3 targetVelocity, float trackerSpeed, out Vector3 interceptPos)
    {
        interceptPos = targetPos; // assume we will aim for the target's pos directly
        float targetSpeedSqr = targetVelocity.sqrMagnitude;

        if (targetSpeedSqr < FLT_EPSILON)
            return true; // target is not moving

        Vector3 vecToTracker = trackerPos - targetPos;
        float distanceToTracker = vecToTracker.magnitude;
        if (distanceToTracker < FLT_EPSILON)
            return true; // already at the target's position

        float a = trackerSpeed * trackerSpeed - targetSpeedSqr; 
        float dotTrackerAndVelocity = Vector3.Dot(vecToTracker.normalized, targetVelocity.normalized);
        if (a < 0.0f && dotTrackerAndVelocity < 0.0f)
            // the target's speed is greater than the tracker's speed and the target is moving away from the tracker,
            // which means that the tracker will not be able to intercept the target
            return false; 

        float b = 2.0f * Vector3.Dot(targetVelocity, vecToTracker);
        float c = -distanceToTracker * distanceToTracker;

        float t1 = 0.0f;
        float t2 = 0.0f;
        QuadraticSolver(a, b, c, out t1, out t2);
        float timeToIntercept = 0.0f;
        if (t1 > 0.0f && t2 > 0.0f)
            timeToIntercept = Mathf.Min(t1, t2); // both values positive, take the smaller one
        else
            timeToIntercept = Mathf.Max(t1, t2); // one value negative, take the bigger (positive) one

        interceptPos = targetPos + targetVelocity * timeToIntercept;
        //Debug.DrawLine(trackerPos, interceptPos, Color.red);
        return true;
    }


}

